/*-----------------------------------------------------------------------

File  : utilities.c

Author: Stephan Schulz

Contents
 
  Implementation for some simple utility functions

  Copyright 1998-2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Sat Oct 26 23:48:48 CEST 2002
    New

-----------------------------------------------------------------------*/

#include "utilities.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

/* Allocate memory, exit with an error-message, if no memory 
   available */ 

void* SecureMalloc(size_t size)
{
   void* res = malloc(size);

   if(!res)
   {
      printf("malloc() failure -- out of memory?");
      exit(EXIT_FAILURE);
   }
   return res;
 }

/* Same thing, just using realloc instead */

void *SecureRealloc(void* ptr, size_t size)
{
   void* res = realloc(ptr,size);

   if(!res)
   {
      printf("realloc() failure -- out of memory?");
      exit(EXIT_FAILURE);
   }
   return res;
 }

/* Read one line of characters from stdin and return it. Always
   include a newline, even if the last line has none. Memory is
   malloc()ed, and must be free()d by the caller. If there is no line
   to read, return NULL */

char *ReadLine()
{
   char* buffer;
   int size = INITIAL_BUFF_SIZE, c, count = 0;

   c = getchar();
   if(c==EOF)
   {
      return NULL;
   }
   buffer = SecureMalloc(size*sizeof(char));

   while(c!='\n' && c!=EOF)
   {
      buffer[count++] = c;
      if(count == size-1) /* Allow for '0' */
      {
	 size *= 2;
	 buffer = SecureRealloc(buffer, size);
      }
      c = getchar();
   }
   buffer[count++]='\n';
   buffer[count]='\0';
   
   return buffer;
}

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


